#
# Try to find the libraw library and include path.
# Once done this will define
#
# LIBRAW_FOUND
# LIBRAW_INCLUDE_PATH
# LIBRAW_LIBRARY
# credits to http://code.google.com/p/nvidia-texture-tools/source/browse/trunk/cmake/FindFreeImage.cmake?r=580

IF(WIN32)
        FIND_PATH( LIBRAW_INCLUDE_PATH libraw.h
                                PATH_SUFFIXES "libraw"
				ENV PATH
                DOC "The directory where libraw.h resides")
        FIND_LIBRARY( LIBRAW_LIBRARY
                NAMES libraw
				ENV PATH
				PATH_SUFFIXES 
                                "lib"
                                "libraw"
                                "libraw/.libs"
                                "buildfiles/x64/release"
                                "buildfiles/x64/debug"
                                "buildfiles/release"
                                "buildfiles/debug"
                DOC "The libraw library")
    set ( MY_LIBS
            ${MY_LIBS}
            "ws2_32")
ELSE(WIN32)
        FIND_PATH( LIBRAW_INCLUDE_PATH libraw.h
	        PATH_SUFFIXES
		libraw
		PATHS
                /usr/include
                /usr/local/include
                /sw/include
                /opt/local/include
                DOC "The directory where libraw.h resides")
        FIND_LIBRARY( LIBRAW_LIBRARY
                NAMES libraw libraw.so
                PATHS
                /usr/lib/x86_64-linux-gnu
                /usr/lib64
                /usr/lib
                /usr/local/lib64
                /usr/local/lib
                /sw/lib
                /opt/local/lib
                DOC "The libraw library")

        if(LIBRAW_LIBRARY)
            get_filename_component(LIBRAW_LIBRARY_DIR ${LIBRAW_LIBRARY} PATH)
            find_file(_raw_pkg_config_file
                    NAMES "libraw.pc"
                    HINTS
                    ${LIBRAW_LIBRARY_DIR}
                    ${LIBRAW_LIBRARY_DIR}/pkgconfig
                    PATH_SUFFIXES
                    pkgconfig
                    PATHS ENV PATH)

            if(EXISTS ${_raw_pkg_config_file})
                file(STRINGS "${_raw_pkg_config_file}" _RAW_version REGEX "^Version: .*$")
                string(REGEX REPLACE "^.*Version: ([0-9]+).*$" "\\1" RAW_VERSION_MAJOR "${_RAW_version}")
                string(REGEX REPLACE "^.*Version: [0-9]+\\.([0-9]+).*$" "\\1" RAW_VERSION_MINOR  "${_RAW_version}")
                string(REGEX REPLACE "^.*Version: [0-9]+\\.[0-9]+\\.([0-9]+).*$" "\\1" RAW_VERSION_PATCH "${_RAW_version}")
                set(RAW_VERSION_STRING "${RAW_VERSION_MAJOR}.${RAW_VERSION_MINOR}.${RAW_VERSION_PATCH}")
            else(EXISTS ${_raw_pkg_config_file})
                set(RAW_VERSION_STRING "")
            endif(EXISTS ${_raw_pkg_config_file})
        else()
            set(_raw_pkg_config_file "")
        endif()
ENDIF(WIN32)


SET(LIBRAW_LIBRARIES ${LIBRAW_LIBRARY})

IF (LIBRAW_INCLUDE_PATH AND LIBRAW_LIBRARY)
        SET( LIBRAW_FOUND TRUE CACHE BOOL "Set to TRUE if LIBRAW is found, FALSE otherwise")
ELSE (LIBRAW_INCLUDE_PATH AND LIBRAW_LIBRARY)
        SET( LIBRAW_FOUND FALSE CACHE BOOL "Set to TRUE if LIBRAW is found, FALSE otherwise")
ENDIF (LIBRAW_INCLUDE_PATH AND LIBRAW_LIBRARY)

MARK_AS_ADVANCED(
        LIBRAW_FOUND
        LIBRAW_LIBRARY
        LIBRAW_LIBRARIES
        LIBRAW_INCLUDE_PATH)

if(NOT WIN32)
	set(CMAKE_CXX_FLAGS "${CMAKE_CXX_FLAGS}  -Wdeprecated-declarations")
endif()

set ( MY_LIBS
  ${MY_LIBS}
  ${LIBRAW_LIBRARIES}
)

include_directories (
  "${LIBRAW_INCLUDE_PATH}/../"
  "${LIBRAW_INCLUDE_PATH}"
)

if (WIN32)
	## otherwise mingw32 won't link:
	#add_definitions("-DFREEIMAGE_LIB")
	#add_definitions("-DFREEIMAGE_EXPORTS")
	
    get_filename_component(LIBRAW_LIB_DIR ${LIBRAW_LIBRARY} DIRECTORY)
    file(GLOB_RECURSE DLL_FILES "${LIBRAW_LIB_DIR}/*.dll")
    INSTALL(FILES ${DLL_FILES}
        DESTINATION bin
		COMPONENT main
    )
endif(WIN32)