#
# Try to find the Libsolid library and include path.
# Once done this will define
#
# LIBSOLID_FOUND
# LIBSOLID_INCLUDE_PATH
# LIBSOLID_LIBRARY

IF(UNIX)
  FIND_PATH( LIBSOLID_INCLUDE_PATH solidnamespace.h
    /usr/include
    /usr/include/solid
    /usr/local/include
    /usr/local/include/solid
    /sw/include
    /sw/include/solid
    /opt/local/include
    /opt/local/include/solid
    DOC "The directory where solidnamespace.h resides")

  FIND_LIBRARY( LIBSOLID_LIBRARY
    NAMES libsolid solid
    PATHS
    /usr/lib64
    /usr/lib
    /usr/local/lib64
    /usr/local/lib
    /sw/lib
    /opt/local/lib
    DOC "The Solid library")
ENDIF (UNIX)

IF (LIBSOLID_INCLUDE_PATH AND LIBSOLID_LIBRARY)
  SET( LIBSOLID_FOUND TRUE CACHE BOOL "Set to TRUE if libsolid is found, FALSE otherwise")
ENDIF (LIBSOLID_INCLUDE_PATH AND LIBSOLID_LIBRARY)

MARK_AS_ADVANCED(
        LIBSOLID_FOUND
        LIBSOLID_LIBRARY
        LIBSOLID_INCLUDE_PATH)

if(UNIX AND NOT LIBSOLID_FOUND)
  message(WARNING "LIBSOLID not found, KDE users won't be able to import files directly from the camera.")
  message("For Ubuntu (tested for 12.10), the kde libsolid header files come with kdelibs5-dev")
elseif(UNIX)
  message("Found libsolid, using it for possible KDE camera imports")
  add_definitions("-DFOUND_LIB_SOLID")
  set ( MY_LIBS
    ${MY_LIBS}
    ${LIBSOLID_LIBRARY}
    )

  if (NOT ${LIBSOLID_INCLUDE_PATH} STREQUAL "/usr/include/solid")
    include_directories (
      "${LIBSOLID_INCLUDE_PATH}/../"
      )
  endif()
endif(UNIX AND NOT LIBSOLID_FOUND)