/***********************************************************************
 * This file is part of module_imageviewer.
 *
 * module_imageviewer is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * module_imageviewer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with module_imageviewer.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2015, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#ifndef IMAGEVIEWERLOADER_H
#define IMAGEVIEWERLOADER_H

#include <QThread>
#include <QString>
#include <QMutex>
#include <QWaitCondition>

#include "smartpointers.h"

class ImageViewerLoader : public QThread
{
    Q_OBJECT
public:
    explicit ImageViewerLoader(QObject *parent = 0);

    ~ImageViewerLoader();

    void deleteInstance();

    void addTarget(const QString& target2,
                   const bool highPriority = false,
                   const bool noStart = false);

    void clear();

public slots:
    void startComputation();
signals:
    void doneLoading(const MiscTools::pQImage&, const QString&);

protected:
    void run();


private:

    QMutex _fileListMutex;
    QWaitCondition _condition;
    QStringList _files;
    const QSize _imSize;
    bool _stopped;
    bool _abortCurrentStartNext;
    bool _abortAndLoadNext;
    QString _current;
};

#endif // IMAGEVIEWERLOADER_H
