/***********************************************************************
 * This file is part of module_imageviewer.
 *
 * module_imageviewer is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * module_imageviewer is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with module_imageviewer.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2015, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#ifndef VIEWIMAGEDIALOG_H
#define VIEWIMAGEDIALOG_H

/******************
  * This is based on Qt's Image Viewer Example
  */

#include <QVideoSurfaceFormat>
#include <QGraphicsVideoItem>
#include <QLabel>
#include <QDialog>
#include <QScrollBar>
#include <QWidget>
#include <QTranslator>
#include <QGraphicsView>
#include <QGraphicsScene>
#include <QGraphicsPixmapItem>
#include <QGraphicsTextItem>
#include <QToolButton>
#include <QSlider>
#include <QGroupBox>
#include <QTimer>
#include <QGraphicsProxyWidget>
#include <QTransform>
#include <QMediaPlayer>

#include "imagepreloader.h"

namespace Ui {
    class ViewImageDialog;
}

class ViewImageDialog : public QDialog
{
    Q_OBJECT

    enum DisplayMode
    {
        DisplayMode_Text,
        DisplayMode_Video,
        DisplayMode_Image
    };

public:
    static ViewImageDialog* instance(QWidget *parent = 0);
    static void closeInstance();
    ~ViewImageDialog();

    void setRawFormats(const QStringList& rawList);

    ImageDataPtr getFileInfoPtr(const int position);
    int getNumFilesOverall() const;
    ImageDataPtr getFileInfoPtrOverallList(const int position);

signals:
    void newImageLabel();

    void fileOverlayChanged();

public slots:
    void loadPosition(const int position=-1);

    void setImages(const QList<ImageDataPtr>&,
                   const int position = 0,
                   const int basenameLength = 0);

    void loadSettings();
    void resetSettings();

    bool eventFilter(QObject *obj, QEvent *event);

protected slots:
    virtual void _toggleFullscreen();
	
    void zoomIn();
    void zoomOut();
    void normalSize();
    void fitToWindowToggle();
    void fitToWindow(QGraphicsItem *item);

    void on_toolButton_zoomIn_clicked();

    void on_toolButton_zoomOut_clicked();

    void on_toolButton_zoomReal_clicked();

    void on_toolButton_zoomWindow_clicked();

    void on_toolButton_toStart_clicked();

    void on_toolButton_previous_clicked();

    void on_toolButton_next_clicked();

    void on_toolButton_toEnd_clicked();

    void on_pushButton_leave_clicked();

    //void _updateImageLabel();

    void _doneLoading(const QString& filepath);

    void on_checkBox_hideRaw_toggled(bool checked);

    void _contextMenuEvent(const QPoint &pos);

    void _updateFileOverlay(const bool onlyPos = true,
                            const QString text = "");

    void on_toolButton_fullscreen_clicked();

    void on_toolButton_rotLeft_clicked();

    void on_toolButton_rotRight_clicked();

    void on_toolButton_whatsthis_clicked();

    void on_spinBox_number_valueChanged(int arg1);

    void playerStateChanged(const QMediaPlayer::State &state);
    void mediaStatusChanged(const QMediaPlayer::MediaStatus& status);
    void positionChanged(const qint64);
    void durationChanged(const qint64);
    void handleError(const QMediaPlayer::Error);
    void playPauseToggled();
    void muteToggled();
    void sliderPositionChange(const int);
    void disappearControls();

    void _displayVideo();

    void  setVideoControlsVisible(const bool visible);

	void loadVideo();

protected:
	
    void _showPixmap(const QString& filepath);

    void closeEvent ( QCloseEvent *);

    void resizeEvent(QResizeEvent* /* event */);

    void scaleImage(double factor);

    void keyPressEvent(QKeyEvent *e);

    //void _computeFitToWindowScale();


    double _scaleFactor;

    QAction *openAct;
    QAction *exitAct;
    QAction *zoomInAct;
    QAction *zoomOutAct;
    QAction *normalSizeAct;
    QAction *fitToWindowAct;
    QAction *nextImageAct;
    QAction *lastImageAct;
    QAction *prevImageAct;
    QAction *firstImageAct;
    QAction *rotLeftAct;
    QAction *rotRightAct;
    Ui::ViewImageDialog *ui;
    //QLabel* _imageLabel;
    QAction *fullScreenAct;

    int _currentPosition;

    static ViewImageDialog* _instance;

    MiscTools::pQPixmap _originalPixmap;

    // QPixmap _first;
    // QPixmap _last;
    // QVector<QPixmap> _cached;

    int _basenameLength;

    bool _reset;

    ImagePreloader _loader;
    int _currNumImages;

    bool _fullScreen;

    bool _fileOverlayActive;

    QLabel* _filenameOverlay;
    QFrame* _videoControlsGroup;
    QToolButton* _playButton;
    QToolButton* _muteButton;
    QSlider* _videoPositionSlider;
    QSlider* _videoVolumeSlider;
    //QGraphicsProxyWidget* _groupProxy;
    QTimer _controlsDisappearTimer;
    int _timeToDisappear;

    QByteArray _geometry;

    explicit ViewImageDialog(QWidget *parent = 0);

    QMediaPlayer * _videoPlayer;
    QString _lastVideoPath;
    QGraphicsScene * _scene;
    QGraphicsTextItem* _textItem;
    QGraphicsPixmapItem*  _pixmapItem;
    QGraphicsVideoItem* _videoItem;
    DisplayMode _currentDisplayMode;
    QTransform _lastTransformation;
    QRectF _lastSceneRect;

    void _displayText(const ImageDataPtr image,
                      const QString& text,
                      const bool updateStatus = true,
                      const bool hideVideo = true);
    void _displayImage();

    QMutex _protectPixmapItem;

	/** 
	 * since immediately playing does not work, save filename and play
	 * after processevent
	 */
	QString _nextVideo;
};





#endif // VIEWIMAGEDIALOG_H
