/***********************************************************************
 * This file is part of module_misc.
 *
 * module_misc is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * module_misc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with module_misc.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2015, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#include "debugtofile.h"

#include <QtDebug>
#include <QFile>
#include <QTextStream>
#include <QDateTime>

namespace MiscModule
{

QString qtDebugFilename;

void customMessageHandler(QtMsgType type,
                          const QMessageLogContext& context,
                          const QString& msg)
{
    const QDateTime dt = QDateTime::currentDateTime();
    const QString dateStr = dt.toString("yyyy-MM-dd hh:mm:ss:zzz");
    QString txt;    
    const QString contextStr = QString("%1:%2(v%3)").arg(context.file).arg(context.line).arg(context.version);
    switch (type) {
    case QtDebugMsg:
        txt = QString("%1, %2; DEBUG: %3").arg(dateStr).arg(contextStr).arg(msg);
        break;
    case QtWarningMsg:
        txt = QString("%1, %2; WARNING: %3").arg(dateStr).arg(contextStr).arg(msg);
        break;
    case QtCriticalMsg:
        txt = QString("%1, %2; CRITICAL: %3").arg(dateStr).arg(contextStr).arg(msg);
        break;
    case QtFatalMsg:
        txt = QString("%1, %2; FATAL: %3").arg(dateStr).arg(contextStr).arg(msg);
        break;
    default:
        txt = QString("%1, %2; UNKNOWN: %3").arg(dateStr).arg(contextStr).arg(msg);
        break;
    }

    QFile outFile(qtDebugFilename);
    outFile.open(QIODevice::WriteOnly | QIODevice::Append);
    QTextStream ts(&outFile);
    ts << txt << endl;
}

void setDebugFilename(const QString &filename)
{
    qtDebugFilename = filename;
}

} // end namespace MiscModule
