/***********************************************************************
 * This file is part of module_misc.
 *
 * module_misc is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * module_misc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with module_misc.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2015, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#include <QIcon>
#include <QPainter>
#include <QPaintEvent>
#include <QDebug>

#include "imagebutton.h"
#include "smartpointers.h"

namespace MiscTools
{

void ImageButton::paintEvent( QPaintEvent* event )
{
    QPushButton::paintEvent(event);
    QPainter p(this);
    p.setClipRect(event->rect());
    QRect rect = event->rect();
    rect.adjust(_margin, _margin, -_margin, -_margin);
    if (this->isEnabled()) {
        _icon.paint( &p, rect, Qt::AlignLeft, QIcon::Normal, QIcon::Off );
    } else {
        _icon.paint( &p, rect, Qt::AlignLeft, QIcon::Disabled, QIcon::Off );
    }

    _iconDone.paint( &p, rect, Qt::AlignRight, QIcon::Normal, QIcon::Off );
}

ImageButton::ImageButton(const int addHorizontal, QWidget *parent)
    : QPushButton(parent)
    , _addHorizontal(addHorizontal)
{
}

QSize ImageButton::minimumSizeHint() const
{
    QSize out = QPushButton::sizeHint();
    out.setWidth(out.width() + out.height() * 3);
    out.setHeight(out.height() + _addHorizontal);
    return out;
}

QSize ImageButton::sizeHint() const
{
    QSize out = minimumSizeHint();
    return out;
}

void ImageButton::setPixmap(const QString& filename, const int margin)
{
    _icon = QIcon();
    pQPixmap p(filename);
    _icon.addPixmap(*p);
    _margin = margin;
}

void ImageButton::setDone(const QString& filename, const int margin)
{
    _iconDone = QIcon();
    pQPixmap p(filename);
    _iconDone.addPixmap(*p);
    _margin = margin;
}

void ImageButton::setHorizontalSizeHintAdd(const int v)
{
    _addHorizontal = v;
}

} // end namespace MiscTools
