/***********************************************************************
 * This file is part of module_misc.
 *
 * module_misc is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * module_misc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with module_misc.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2015, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#ifndef IMAGEFLIP_H
#define IMAGEFLIP_H

#include <QLabel>
#include <QSize>
#include <QPropertyAnimation>

#define ADD_BORDER_TO_IMAGE 1

#include "smartpointers.h"

namespace MiscTools
{

class ImageFlip : public QLabel
{
    Q_OBJECT
    Q_PROPERTY(qreal currentStatus READ currentStatus WRITE setCurrentStatus)

public:
    explicit ImageFlip(QWidget *parent = 0);
    explicit ImageFlip(const pQImage &im, QWidget *parent = 0);
    ~ImageFlip();

    qreal currentStatus() const;
    void setCurrentStatus(const qreal status);

    void setSize(const int);

    virtual QSize minimumSizeHint () const;
    virtual QSize sizeHint () const;

    void setNewImage(const pQImage&);

    void setupAnimation();

    void setAnimationMode(const QEasingCurve curve, const qreal amplitude);

    void setTime(const qint32);

signals:
    
public slots:
    void startAnimation();

private:
    qreal _currentStatus;

    pQImage _orig;
    pQImage _newOrig;

    int _viewSize;

    QPropertyAnimation* _animation;

    QEasingCurve _currEasingCurve;
    qreal _easingAmplitude;
    qint32 _time;
};

} // end namespace MiscTools
#endif // IMAGEFLIP_H
