/***********************************************************************
 * This file is part of module_randomimageframe.
 *
 * module_randomimageframe is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * module_randomimageframe is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with module_randomimageframe.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2015, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#ifndef RANDOMIMAGELIST_H
#define RANDOMIMAGELIST_H

#include <QFrame>
#include <QEasingCurve>
#include <QTimer>

#include "imagedata.h"
#include "imageflip.h"

struct ImageChange
{
    QEasingCurve::Type type;
    qint32 time;
    qint32 amplitude;

    ImageChange(QEasingCurve::Type type, qint32 time, qint32 amplitude)
        : type(type)
        , time(time)
        , amplitude(amplitude)
    {}
};

class RandomImageList : public QFrame
{
    Q_OBJECT
public:
    explicit RandomImageList(const qint32 thumbsize = 128,
                             const qint32 timeToChange = 5000,
                             const QList<ImageChange> &modes = QList<ImageChange>(),
                             QWidget *parent = 0);
    ~RandomImageList();
    
    void setTimeToChange(const qint32);
    void setThumbnailsize(const qint32);
    void setChangeModes(const QList<ImageChange>&);
    void clearImages();
    void addImage(const ImageDataPtr&);

    virtual QSize minimumSizeHint () const;
    virtual QSize sizeHint () const;

    void addWidget();
    void delWidget();
    qint32 getNumPossible() const;
signals:
    
public slots:
    void startAnimation();
    void stopAnimation();
    void runAnimation();

protected:
    void resizeEvent(QResizeEvent *);

private:
    qint32 _thumbsize;
    qint32 _timeToChange;
    QList<ImageChange> _changeModes;

    QList<ImageDataPtr> _images;
    QList<MiscTools::ImageFlip*> _imageWidgets;
    QVector<qint32> _currInd;

    qint32 _lastImageIndex;
    qint32 _lastViewIndex;
    bool _started;
    QTimer _timer;
};

#endif // RANDOMIMAGELIST_H
