/***********************************************************************
 * This file is part of module_thumbnailcache.
 *
 * module_thumbnailcache is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * module_thumbnailcache is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with module_thumbnailcache.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2015, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#ifndef THUMBNAILLOADER_H
#define THUMBNAILLOADER_H

#include <QThread>
#include <QStringList>
#include <QWaitCondition>
#include <QMutex>

#include "imagedata.h"
#include "fastimagethumbnail.h"

class ThumbnailLoader : public QThread
{
    Q_OBJECT

public:
    ThumbnailLoader(QObject *parent = 0);
    ~ThumbnailLoader();

    void stop();

    void computeThumbnails(const QList<ImageDataPtr>& newFiles,
                           const bool fastOnly=false);

    void setLocationPath(const QString& path);

    static QStringList returnRawFormats();

    void setPause(const bool pause);

    static QMutex pauseMutex;

signals:
    void loadedThumbnails(const QFileInfo& file,
                          const MiscTools::pQImage& thumbnail);

    void errorLoadingThumb(const QFileInfo& file);

    void doneFast();

protected:
    void run();

private:    

    QList<ImageData> _files;

    volatile bool _stopped;

    bool _fastOnly;

    QWaitCondition _condition;
    QMutex _stringListMutex;

    FastImageThumbnail* _thumbnailCache;

};

#endif // THUMBNAILLOADER_H
