/***********************************************************************
 * This file is part of pivot.
 *
 * pivot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pivot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with pivot.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2015, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#include "copyassistant.h"

#include <QMessageBox>

CopyAssistant::CopyAssistant(QObject *parent)
    : QObject(parent)
    , _state(CopyStateStart)
{
}

void CopyAssistant::restart(QWidget* parent)
{
    _state = CopyStateStart;
    doneNextStep(parent);
}

void CopyAssistant::doneNextStep(QWidget* parent)
{

    switch(_state) {
    case CopyStateStart:
    {
        QMessageBox::StandardButton answer =
                QMessageBox::question(parent,
                                      tr("Load files"),
                                      tr("This is the copy assistent. "
                                         "It can be cancelled any time, relaunching it will continue at the same point.\n\n"
                                         "First of all, you have to load the files to be copied.\n"
                                         "Do you want to load a directory now ?"),
                                      QMessageBox::No | QMessageBox::Yes);
        if (answer == QMessageBox::Yes) {
            _state = CopyStateLoading;
            emit loadFiles();
        }
        break;
    }
    case CopyStateLoading:
    {
        QMessageBox::StandardButton answer =
                QMessageBox::question(parent,
                                      tr("Adapt Times"),
                                      tr("Do you have images/videos from different cameras and"
                                         " do you want to adapt the different times now ?"),
                                      QMessageBox::No
                                      | QMessageBox::Cancel
                                      | QMessageBox::Yes);
        if (answer == QMessageBox::Yes) {
            _state = CopyStateAdaptTimes;
            emit adaptTimes();
        } else if (answer == QMessageBox::No) {
            _state = CopyStateAdaptTimes;
            doneNextStep(parent);
        }
        break;
    }
    case CopyStateAdaptTimes:
    {
        QMessageBox::StandardButton answer =
                QMessageBox::question(parent,
                                      tr("Adapt Times"),
                                      tr("Do you now want to manage the albums for the loaded files ?"),
                                      QMessageBox::No
                                      | QMessageBox::Cancel
                                      | QMessageBox::Yes);
        if (answer == QMessageBox::Yes) {
            _state = CopyStateAlbums;
            emit manageAlbums();
        }else if (answer == QMessageBox::No) {
            _state = CopyStateAlbums;
            doneNextStep(parent);
        }
        break;
    }
    case CopyStateAlbums:
    {
        QMessageBox::StandardButton answer =
                QMessageBox::question(parent,
                                      tr("Map Directories"),
                                      tr("Do you now want to create or keep sub-directories from any source sub-directory (e.g. for separating different photographers) ?"),
                                      QMessageBox::No
                                      | QMessageBox::Cancel
                                      | QMessageBox::Yes);
        if (answer == QMessageBox::Yes) {
            _state = CopyStateKeepSubdirs;
            emit keepSubdirectories();
        }else if (answer == QMessageBox::No) {
            _state = CopyStateKeepSubdirs;
            doneNextStep(parent);
        }
        break;
    }
    case CopyStateKeepSubdirs:
    {
        QMessageBox::StandardButton answer =
                QMessageBox::question(parent,
                                      tr("Preview copying"),
                                      tr("Do you now want to see a preview of the copy result (text list) ?"),
                                      QMessageBox::No
                                      | QMessageBox::Cancel
                                      | QMessageBox::Yes);
        if (answer == QMessageBox::Yes) {
            _state = CopyStateCopyPreview;
            emit copyPreview();
        }else if (answer == QMessageBox::No) {
            _state = CopyStateCopyPreview;
            doneNextStep(parent);
        }
        break;
    }
    case CopyStateCopyPreview:
    {
        QMessageBox::StandardButton answer =
                QMessageBox::question(parent,
                                      tr("Start copying"),
                                      tr("Do you now want to start copying ?"),
                                      QMessageBox::No
                                      | QMessageBox::Yes);
        if (answer == QMessageBox::Yes) {
            _state = CopyStateStart;
            emit copy();
        }
        break;
    }
    default:
        break;
    }
}
