/***********************************************************************
 * This file is part of pivot.
 *
 * pivot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pivot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with pivot.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2015, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#include "directorydelegate.h"
#include <QComboBox>

DirectoryDelegate::DirectoryDelegate(QObject *parent) :
    QItemDelegate(parent)
{
    _directories.push_back(".");
}


void DirectoryDelegate::paint(QPainter *painter,
                          const QStyleOptionViewItem &option,
                          const QModelIndex &index) const
{
//    if (index.column() == 1 || index.column() == 2) {
//        QDateTime val(QDateTime::fromTime_t(index.model()->data(index, Qt::DisplayRole).toUInt()));
//        QStyleOptionViewItem myOption = option;
//        myOption.displayAlignment = Qt::AlignRight | Qt::AlignVCenter;
//        drawDisplay(painter, myOption, myOption.rect, val.toString(Qt::DefaultLocaleShortDate));
//        drawFocus(painter, myOption, myOption.rect);
//    } else {
        QItemDelegate::paint(painter, option, index);
//    }
}

QWidget * DirectoryDelegate::createEditor(QWidget *parent,
                                      const QStyleOptionViewItem &option,
                                      const QModelIndex &index) const
{
    if (index.column() == 0 ) {
        QComboBox *combobox = new QComboBox(parent);
        for (int i=0; i<_directories.size(); i++) {
            combobox->addItem(_directories[i]);
        }
        connect(combobox, SIGNAL(editingFinished()),
                this, SLOT(commitAndCloseEditor()));
        return combobox;
    } else {
        return QItemDelegate::createEditor(parent, option, index);
    }
}


void DirectoryDelegate::commitAndCloseEditor()
{
    QComboBox *editor = qobject_cast<QComboBox*>(sender());
    emit commitData(editor);
    emit closeEditor(editor);
}

void DirectoryDelegate::setEditorData(QWidget *editor,
                                  const QModelIndex &index) const
{
    if (index.column() == 0) {
        QString value = index.model()->data(index, Qt::DisplayRole).toString();
        QComboBox * combobox =  qobject_cast<QComboBox*>(editor);
        combobox->setCurrentIndex(combobox->findText( value, Qt::MatchExactly ));
    } else {
        QItemDelegate::setEditorData(editor, index);
    }
}

void DirectoryDelegate::setModelData(QWidget *editor,
                                 QAbstractItemModel *model,
                                 const QModelIndex &index) const
{
    if (index.column() == 0) {
        QComboBox * combobox =  qobject_cast<QComboBox*>(editor);
        model->setData(index, combobox->currentText());
    } else {
        QItemDelegate::setModelData(editor, model, index);
    }
}
