/***********************************************************************
 * This file is part of pivot.
 *
 * pivot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pivot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with pivot.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2015, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#include <QSettings>
#include <QDebug>
#include <QMessageBox>
#include <QHBoxLayout>
#include <QSpacerItem>
#include <QVector>
#include <QTimer>
#include <QMouseEvent>
#include <QFile>
#include <QDir>
#include <QKeyEvent>
#include <QWhatsThis>

#include <flowlayout.h>
#include <viewimagedialogextended.h>

#include "eventselectionassistant.h"
#include "ui_eventselectionassistant.h"
#include "settings.h"
#include "imageviewmodel.h"
#include "albumdelegate.h"
#include "tools.h"

QDataStream &operator<<(QDataStream &out, const PhotoAlbum &album)
{

    out << album.name << album.from << album.to << album.ID;
    return out;
}
QDataStream &operator>>(QDataStream &in, PhotoAlbum & album)

{
    in >> album.name >> album.from >> album.to >> album.ID;

//    qDebug() << album.name  << " "
//             << from  << "(" << album.from.toTime_t() << ")"
//             << to << "(" << album.to.toTime_t() << ")"
//             << album.ID;
    return in;
}

qint64 EventSelectionAssistant::_lastID = 0;

EventSelectionAssistant::EventSelectionAssistant(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::EventSelectionAssistant)
    , _init(true)
    , _dontUndo(false)
    , _undoPosition(0)
    , _infoHasBeenShown(false)
    , _currLimit(100)
{
    ui->setupUi(this);

    _selectSizes(ui->spinBox_thumbnailSize->value());

    this->setWindowTitle(tr("Album Assistant"));

    FlowLayout * insideL = new FlowLayout;
    insideL->setContentsMargins(5,0,0,0);
    ui->scrollAreaWidgetContents_inside->setLayout(insideL);
    FlowLayout* beforeL= new FlowLayout;
    insideL->setContentsMargins(5,0,0,0);
    ui->scrollAreaWidgetContents_before->setLayout(beforeL);
    FlowLayout* afterL = new FlowLayout;
    insideL->setContentsMargins(5,0,0,0);
    ui->scrollAreaWidgetContents_after->setLayout(afterL);

    ui->tableWidget_events->setColumnCount(4);
    ui->tableWidget_events->setHorizontalHeaderLabels(QStringList() << tr("Short Album Description") << tr("Start Date/Time") << tr("End Date/Time"));

    ui->tableWidget_events->setSelectionMode(QAbstractItemView::SingleSelection);
    ui->tableWidget_events->setSelectionBehavior (QAbstractItemView::SelectRows);
    ui->tableWidget_events->setColumnHidden(3, true);
    ui->tableWidget_events->horizontalHeader()->setSectionResizeMode(QHeaderView::Stretch);

    ui->tableWidget_events->setItemDelegate(new AlbumDelegate());

    loadSettings();

    connect(ui->tableWidget_events, SIGNAL(cellChanged(int,int)),
            this, SLOT(updateUndoList()));

    connect(ui->tableWidget_events, SIGNAL(itemSelectionChanged()),
            this, SLOT(itemSelectionChanged()));


    // use event filter to deselect table entries, when starting to edit
    ui->lineEdit_newEventDescription->installEventFilter(this);
    ui->dateTimeEdit_newEventFrom->installEventFilter(this);
    ui->dateTimeEdit_newEventTo->installEventFilter(this);

    // capture middle click
    ui->tableWidget_events->installEventFilter(this);
    //FIXIT ui->tableWidget_events->horizontalHeader()->setResizeMode(QHeaderView::Stretch);

    _init = false;

    //ui->pushButton_addAlbum->setFocus();
    ui->lineEdit_newEventDescription->setFocus();

    ui->lineEdit_newEventDescription->installEventFilter(this);

}

void EventSelectionAssistant::resetLimitOutside()
{
    //ui->checkBox_limitOutside->setChecked(settings.value("eventAssistant_showLimitOutside", true).toBool());
    ui->checkBox_limitOutside->setChecked(true);
}

void EventSelectionAssistant::loadSettings()
{
    QSettings settings(tr(IMAGE_ORGANIZATION_ORG), tr(IMAGE_ORGANIZATION_APP));

    ui->checkBox_showFixed->setChecked(settings.value("eventAssistant_showFixedNumber", true).toBool());
    ui->spinBox_numPics->setValue(settings.value("eventAssistant_showFixedNumberValue", 10).toInt());
    ui->spinBox_numPics->setEnabled(ui->checkBox_showFixed->isChecked());
    ui->checkBox_random->setChecked(settings.value("eventAssistant_showRandomSelection", false).toBool());
    ui->checkBox_random->setEnabled(ui->checkBox_showFixed->isChecked());
    if (!ui->checkBox_showFixed->isChecked()) {
        ui->toolButton_reselect->setEnabled(false);
    }
    ui->spinBox_thumbnailSize->setValue(settings.value("eventAssistant_thumbnailSize", 128).toInt());

    ui->spinBox_numOutside->setValue(settings.value("eventAssistant_numLimitOutside", 10).toInt());
    resetLimitOutside();
    ui->tableWidget_events->setColumnWidth(0, settings.value("eventAssistant_column0Width", 450).toInt());
    ui->tableWidget_events->setColumnWidth(1, settings.value("eventAssistant_column1Width", 200).toInt());
    ui->tableWidget_events->setColumnWidth(2, settings.value("eventAssistant_column2Width", 200).toInt());

    ui->splitter->restoreState(settings.value("album_splitter_state").toByteArray());
    if (settings.value("album_splitter2_state").toByteArray().length() == 0) {
        ui->splitter_2->setSizes(QList<int>() << 300 << 200);
    } else {
        ui->splitter_2->restoreState(settings.value("album_splitter2_state").toByteArray());
    }
    this->restoreGeometry(settings.value("album_dialog_geometry").toByteArray());
}


void EventSelectionAssistant::_selectSizes(const int)
{
    //const int height = thumbsize;
    //QFontMetrics fm( ui->label->font() );

    //ui->groupBox_imagesAfter->setMinimumSize(0, height + fm.height());
    //ui->groupBox_imagesBefore->setMinimumSize(0, height + fm.height());
    //ui->groupBox_imagesCurrent->setMinimumSize(0, height + fm.height());


}

EventSelectionAssistant::~EventSelectionAssistant()
{
    _writeAlbumsToFile();

    QSettings settings(tr(IMAGE_ORGANIZATION_ORG), tr(IMAGE_ORGANIZATION_APP));
    settings.setValue("eventAssistant_showFixedNumber",ui->checkBox_showFixed->isChecked());
    settings.setValue("eventAssistant_showFixedNumberValue",ui->spinBox_numPics->value());
    settings.setValue("eventAssistant_showRandomSelection",ui->checkBox_random->isChecked());
    settings.setValue("eventAssistant_thumbnailSize",ui->spinBox_thumbnailSize->value());
    settings.setValue("eventAssistant_showLimitOutside",ui->checkBox_limitOutside->isChecked());
    settings.setValue("eventAssistant_numLimitOutside",ui->spinBox_numOutside->value());

    settings.setValue("eventAssistant_column0Width", ui->tableWidget_events->columnWidth(0));
    settings.setValue("eventAssistant_column1Width", ui->tableWidget_events->columnWidth(1));
    settings.setValue("eventAssistant_column2Width", ui->tableWidget_events->columnWidth(2));

    settings.setValue("album_splitter_state", ui->splitter->saveState());
    settings.setValue("album_splitter2_state", ui->splitter_2->saveState());
    settings.setValue("album_dialog_geometry", this->saveGeometry());

    delete ui;
}

void EventSelectionAssistant::on_checkBox_showFixed_toggled(bool checked)
{
    if (checked) {
        ui->toolButton_reselect->setEnabled(ui->checkBox_random->isChecked());
    } else {
        ui->toolButton_reselect->setEnabled(false);
    }

    displayDate();
}

bool lessThan(const ImageDataPtr s1,
                             const ImageDataPtr s2)
{
    const ImageDataExtended* tmp1  = dynamic_cast<ImageDataExtended*>(s1.get());
    const ImageDataExtended* tmp2  = dynamic_cast<ImageDataExtended*>(s2.get());

    return tmp1->getTime() == tmp2->getTime() ?
                tmp1->getFileInfo().canonicalFilePath() < tmp2->getFileInfo().canonicalFilePath()
              : tmp1->getTime() < tmp2->getTime();
}


void EventSelectionAssistant::setImageList(const QList<ImageDataPtr>& photoList,
                                           const int baseDirLength)
{
    _baseDirLength = baseDirLength;


    _photoList = photoList;
    if (_photoList.size() > 1) {
        qSort(_photoList.begin(), _photoList.end(), lessThan);
    }

    _setNextUncoveredTime();
}

void EventSelectionAssistant::_setNextUncoveredTime()
{
    QDateTime date;

    // find start date
    _getNextUncoveredDate(date);

    // find end date (complete day or begin of next album)
    QDateTime date2 = date;
    date2.setTime(QTime(23,59,59,0));
    QVector<PhotoAlbum> albums = getAlbums();
    for (int j=0; j<(int)albums.size(); j++) {
        if ( date2.date().toJulianDay() == albums[j].from.date().toJulianDay()
             && date < albums[j].from) {
            date2 = qMin(date2, albums[j].from);
        }
    }

    _currFrom = date;
    _currTo = date2;
    displayDate();
    ui->dateTimeEdit_newEventFrom->setDateTime(date);
    ui->dateTimeEdit_newEventTo->setDateTime(date2);
}

void EventSelectionAssistant::on_pushButton_addAlbum_clicked()
{
    _dontUndo = true;
    if (ui->lineEdit_newEventDescription->text().trimmed().length() == 0) {
        QMessageBox::warning ( this,
                               tr("No text"),
                               tr("Please enter a short description of this event"));
        return;
    }

    ui->dateTimeEdit_newEventFrom->interpretText();
    ui->dateTimeEdit_newEventTo->interpretText();

    if (!ui->dateTimeEdit_newEventFrom->date().isValid()
            || !ui->dateTimeEdit_newEventTo->date().isValid()) {

        QMessageBox::warning ( this,
                               tr("Date invalid"),
                               tr("Please enter valid dates and times for this event"));
        return;
    }

    QVector<PhotoAlbum> tmp = getAlbums();
    PhotoAlbum newAlbum;
    newAlbum.name = ui->lineEdit_newEventDescription->text();
    newAlbum.from = ui->dateTimeEdit_newEventFrom->dateTime();
    newAlbum.to = ui->dateTimeEdit_newEventTo->dateTime();
    newAlbum.ID = ++_lastID;
    tmp.push_back(newAlbum);
    _setAlbums(tmp);


    _lastSkippedTo = newAlbum.from;
    _setNextUncoveredTime();
    ui->lineEdit_newEventDescription->setText("");

    _dontUndo = false;
    updateUndoList();
}

bool EventSelectionAssistant::_getNextUncoveredDate(QDateTime& newDate)
{
    QVector<PhotoAlbum> albums = getAlbums();

    newDate = QDateTime::currentDateTime();
    QDateTime oldest;
    bool found = false;
    for (int i=0; i<(int)_photoList.size(); i++) {
        bool covered = false;

        QDateTime currDate = dynamic_cast<ImageDataExtended*> (_photoList[i].get())->getTime() ;
        oldest = currDate > oldest && _lastSkippedTo < currDate ? currDate : oldest;
        if (!currDate.isValid())
            continue;

        if (_lastSkippedTo.isValid() && currDate < _lastSkippedTo) {
            covered = true;
        } else {
            for (int j=0; j<(int)albums.size(); j++) {
                if ( (currDate >= albums[j].from && currDate < albums[j].to) ) {
                    covered = true;
                    break;
                }
            }
        }
        if (!covered) {
            newDate = currDate < newDate ? currDate : newDate;
            found = true;
        }
    }
    if (!found) {
        if (_lastSkippedTo.isValid()) {
            newDate = _lastSkippedTo.addMSecs(1);
        } else {
            newDate = oldest.isValid() ? oldest.addSecs(1) : QDateTime::currentDateTime();
        }
    }
    _lastSkippedTo = newDate;
    return found;
}


void EventSelectionAssistant::onLoadedThumbnails(const QFileInfo info, const MiscTools::pQImage thumb)
{
    for (int i=0; i<(int)_photoList.size(); i++) {
        ImageDataExtended tmp = *dynamic_cast<ImageDataExtended*> (_photoList[i].get());
        if (info.canonicalFilePath() == tmp.getFileInfo().canonicalFilePath()) {
            tmp.setThumbnail(thumb);
            tmp.setHasThumbnail(true);
            *dynamic_cast<ImageDataExtended*> (_photoList[i].get()) = tmp;
            _checkImageWidget(tmp);
            break;
        }
    }
}


void EventSelectionAssistant::onErrorThumbnails(const QFileInfo info)
{
    for (int i=0; i<(int)_photoList.size(); i++) {
        ImageDataExtended tmp = *dynamic_cast<ImageDataExtended*> (_photoList[i].get());
        if (info.canonicalFilePath() == tmp.getFileInfo().canonicalFilePath()) {
            tmp.setThumbnail(ImageViewModel::errorThumb);
            tmp.setHasThumbnail(true);
            *dynamic_cast<ImageDataExtended*> (_photoList[i].get()) = tmp;
            _checkImageWidget(tmp);
            break;
        }
    }
}

void EventSelectionAssistant::_checkImageWidget(const ImageDataExtended& search)
{
    for (int i=0; i<(int)_widgetsInside.size(); i++) {
        if (_widgetsInside[i]->getImageDataExtented().getFileInfo().canonicalFilePath()
            == search.getFileInfo().canonicalFilePath())
        {
            _widgetsInside[i]->updateThumbnail(search.getThumbnail());
        }
    }

    for (int i=0; i<(int)_widgetsBefore.size(); i++) {
        if (_widgetsBefore[i]->getImageDataExtented().getFileInfo().canonicalFilePath()
            == search.getFileInfo().canonicalFilePath())
        {
            _widgetsBefore[i]->updateThumbnail(search.getThumbnail());
        }
    }

    for (int i=0; i<(int)_widgetsAfter.size(); i++) {
        if (_widgetsAfter[i]->getImageDataExtented().getFileInfo().canonicalFilePath()
            == search.getFileInfo().canonicalFilePath())
        {
            _widgetsAfter[i]->updateThumbnail(search.getThumbnail());
        }
    }
}

void EventSelectionAssistant::_connectImageWidget(ImageWidget* w)
{
    connect(w, SIGNAL(signalTimeFromForNew(QDateTime)),
            this, SLOT(onTimeFromForNewTriggered(QDateTime)));
    connect(w, SIGNAL(signalTimeToForNew(QDateTime)),
            this, SLOT(onTimeToForNewTriggered(QDateTime)));
    connect(w, SIGNAL(signalTimeFromForTable(QDateTime)),
            this, SLOT(onTimeFromForTableTriggered(QDateTime)));
    connect(w, SIGNAL(signalTimeToForTable(QDateTime)),
            this, SLOT(onTimeToForTableTriggered(QDateTime)));

    connect(this, SIGNAL(tableViewSelected(bool)),
            w, SLOT(setTableSelected(bool)));

    connect(w, SIGNAL(signalViewAllImages(const QString&)),
            this, SLOT(_viewAllImages(const QString&)));
    connect(w, SIGNAL(signalViewCurrentAlbum(const QString&)),
            this, SLOT(_viewCurrentAlbum(const QString&)));

}


void EventSelectionAssistant::displayDate(const bool onlyNotYetCovered)
{
    if (_init) {
        return;
    }

    QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
    const int numPics = ui->spinBox_numPics->value();
    const int thumbSize = ui->spinBox_thumbnailSize->value();
    const bool random = ui->checkBox_random->isChecked();

    _clearImages();

    QList<ImageDataExtended> inside, before, after;
    for (int i=0; i< (int)_photoList.size(); i++) {
        ImageDataExtended tmp = *dynamic_cast<ImageDataExtended*>(_photoList[i].get());
        QDateTime iTime = tmp.getTime();
        if (iTime >= _currFrom && iTime < _currTo) {
            inside.push_back( tmp);
        } else if (iTime < _currFrom) {
            before.push_back( tmp );
        } else {
            after.push_back( tmp );
        }
    }


    const bool albumIsSelected = this->_getSelectedRow(false) >= 0;
    const int stepsize = ui->checkBox_showFixed->isChecked() ? std::max<int>(1,inside.size()/numPics) : 1;
    int k;
    bool last=false;
    if (random && ui->checkBox_showFixed->isChecked()) {
        const int numWanted = ui->spinBox_numPics->value();
        if (numWanted >= inside.size()) {
            for (int i=0; i<inside.size();i++) {
                ImageWidget* newImgWidget = new ImageWidget(inside[i], thumbSize, true, albumIsSelected);
                ui->scrollAreaWidgetContents_inside->layout()->addWidget(newImgWidget);
                _connectImageWidget(newImgWidget);

                _widgetsInside.push_back(newImgWidget);
            }
        } else {
            QVector<int> chosen;
            for (int i=0; i<numWanted;i++) {
                int curr;
                do {
                    curr = rand() % inside.size();
                } while (chosen.indexOf(curr) > 0) ;
                chosen.push_back(curr);

                ImageWidget* newImgWidget = new ImageWidget(inside[curr], thumbSize, true, albumIsSelected);
                ui->scrollAreaWidgetContents_inside->layout()->addWidget(newImgWidget);
                _connectImageWidget(newImgWidget);

                _widgetsInside.push_back(newImgWidget);
            }
        }
    } else {
        for (k=0; k<(int)inside.size(); k+=stepsize) {
            ImageWidget* newImgWidget = new ImageWidget(inside[k], thumbSize, true, albumIsSelected);
            ui->scrollAreaWidgetContents_inside->layout()->addWidget(newImgWidget);
            _connectImageWidget(newImgWidget);
            _widgetsInside.push_back(newImgWidget);
            if (k == inside.size()-1) {
                last=true;
            }
        }
        if (!last && inside.size() > 0) {
            ImageWidget* newImgWidget = new ImageWidget(inside.last(), thumbSize, true, albumIsSelected);
            ui->scrollAreaWidgetContents_inside->layout()->addWidget(newImgWidget);
            _connectImageWidget(newImgWidget);
            _widgetsInside.push_back(newImgWidget);
        }
    }


    const int numOutside = ui->spinBox_numOutside->value();
    const bool limitOutside = ui->checkBox_limitOutside->isChecked();
    const int numPicsOutside = limitOutside ? numOutside : qMax(before.size(), after.size());

    for (int i=0; i < numPicsOutside; i++) {
        const int posBefore = before.size()- (limitOutside ? numOutside : before.size())+i;
        if (posBefore >=0 && posBefore < before.size()) {
            ImageWidget* newImgWidget = new ImageWidget( before[posBefore], thumbSize, false, albumIsSelected);
            ui->scrollAreaWidgetContents_before->layout()->addWidget(newImgWidget);
            _connectImageWidget(newImgWidget);
            _widgetsBefore.push_back(newImgWidget);
        }
        if (i < after.size()) {
            ImageWidget* newImgWidget = new ImageWidget( after[i], thumbSize, false, albumIsSelected);
            ui->scrollAreaWidgetContents_after->layout()->addWidget(newImgWidget);
            _connectImageWidget(newImgWidget);
            _widgetsAfter.push_back(newImgWidget);
        }
    }
    ui->scrollArea->verticalScrollBar()->setSliderPosition(0);
    ui->scrollArea_2->verticalScrollBar()->setSliderPosition(0);
    ui->scrollArea_3->verticalScrollBar()->setSliderPosition(0);

        qApp->restoreOverrideCursor();
}

void EventSelectionAssistant::_clearImages()
{
    for (int i=0; i<(int)_widgetsInside.size(); i++) {
        ui->scrollAreaWidgetContents_inside->layout()->removeWidget(_widgetsInside[i]);
        _widgetsInside[i]->deleteLater();
        _widgetsInside[i] = 0;
    }
    _widgetsInside.clear();

    for (int i=0; i<(int)_widgetsBefore.size(); i++) {
        ui->scrollAreaWidgetContents_before->layout()->removeWidget(_widgetsBefore[i]);
        _widgetsBefore[i]->deleteLater();
        _widgetsBefore[i] = 0;
    }
    _widgetsBefore.clear();

    for (int i=0; i<(int)_widgetsAfter.size(); i++) {
        ui->scrollAreaWidgetContents_after->layout()->removeWidget(_widgetsAfter[i]);
        _widgetsAfter[i]->deleteLater();
        _widgetsAfter[i] = 0;
    }
    _widgetsAfter.clear();

}

void EventSelectionAssistant::on_spinBox_thumbnailSize_editingFinished()
{
    const int val = ui->spinBox_thumbnailSize->value();
    qApp->setOverrideCursor( Qt::WaitCursor );
    for (int i=0; i<(int)_widgetsInside.size(); i++) {
        _widgetsInside[i]->updateThumbnailSize(val);
    }

    for (int i=0; i<(int)_widgetsBefore.size(); i++) {
        _widgetsBefore[i]->updateThumbnailSize(val);
    }

    for (int i=0; i<(int)_widgetsAfter.size(); i++) {
        _widgetsAfter[i]->updateThumbnailSize(val);
    }

    qApp->restoreOverrideCursor();
}

void EventSelectionAssistant::on_spinBox_thumbnailSize_valueChanged(int)
{

}


bool albumLessThan(const PhotoAlbum& a1, const PhotoAlbum& a2)
{
    return a1.from < a2.from;
}

QVector<PhotoAlbum> EventSelectionAssistant::getAlbums()
{
    QVector<PhotoAlbum> out;
    for (int i=0; i<ui->tableWidget_events->rowCount(); i++) {
        PhotoAlbum tmp;
        tmp.name = ui->tableWidget_events->item(i, 0)->data(Qt::DisplayRole).toString();
        tmp.from = QDateTime::fromTime_t(ui->tableWidget_events->item(i, 1)->data(Qt::DisplayRole).toUInt());
        tmp.to = QDateTime::fromTime_t(ui->tableWidget_events->item(i, 2)->data(Qt::DisplayRole).toUInt());
        tmp.ID = ui->tableWidget_events->item(i, 3)->data(Qt::DisplayRole).toInt();
        out.push_back(tmp);
    }

    qStableSort(out.begin(), out.end(), albumLessThan);
    return out;
}

void EventSelectionAssistant::_setAlbums(const QVector<PhotoAlbum>& albumsOrig)
{
    QVector<PhotoAlbum> albums = albumsOrig;
    qStableSort(albums.begin(), albums.end(), albumLessThan);

    const bool copyToUndo = !_dontUndo;
    _dontUndo = true;
    ui->tableWidget_events->setRowCount(albums.size());
    for (int i=0; i<(int)albums.size(); i++) {
        QTableWidgetItem * albumTitle = new QTableWidgetItem(albums[i].name);
        QTableWidgetItem * albumStart = new QTableWidgetItem(QString::number(albums[i].from.toTime_t()));
        QTableWidgetItem * albumEnd = new QTableWidgetItem(QString::number(albums[i].to.toTime_t()));
        QTableWidgetItem * albumId = new QTableWidgetItem(QString::number(albums[i].ID));
        ui->tableWidget_events->setItem(i, 0, albumTitle);
        ui->tableWidget_events->setItem(i, 1, albumStart);
        ui->tableWidget_events->setItem(i, 2, albumEnd);
        ui->tableWidget_events->setItem(i, 3, albumId);
    }

    _dontUndo = false;
    if (copyToUndo) updateUndoList();
}

void EventSelectionAssistant::on_checkBox_random_toggled(bool)
{
    displayDate();
}

void EventSelectionAssistant::on_toolButton_reselect_clicked()
{
    displayDate();
}

void EventSelectionAssistant::on_checkBox_limitOutside_toggled(bool limit)
{
    if (!limit) {
        QList<ImageDataExtended> before, after;
        for (int i=0; i< (int)_photoList.size(); i++) {
            ImageDataExtended tmp = *dynamic_cast<ImageDataExtended*>(_photoList[i].get());
            QDateTime iTime = tmp.getTime();
            if (iTime >= _currFrom && iTime < _currTo) {
                continue;
            } else if (iTime < _currFrom) {
                before.push_back( tmp );
            } else {
                after.push_back( tmp );
            }
        }
        const int n = qMax(before.size(), after.size());
        if (n > (int)_currLimit) {
            QMessageBox::StandardButton answer =
                    QMessageBox::question(this,
                                          tr("Many Images"),
                                          tr("Not limiting then number of not within date span images"
                                             " means loading and displaying %1 thumbnails."
                                             " This may take a while to accomplish."
                                             " Do you really want to enable this ?").arg(n),
                                          QMessageBox::No | QMessageBox::Yes);
            if (answer == QMessageBox::No) {
                ui->checkBox_limitOutside->setChecked(true);
                return;
            }
            _currLimit = n;
        }
    }
    displayDate();
}

void EventSelectionAssistant::on_spinBox_numOutside_valueChanged(int)
{
    displayDate();
}

void EventSelectionAssistant::_on_pushButton_finish_clicked()
{
    _writeAlbumsToFile();
    emit finishedAlbums();
}

void EventSelectionAssistant::on_toolButton_deleteRow_clicked()
{
    const int row = _getSelectedRow();
    if (row < 0) return;

    _dontUndo = true;
    ui->tableWidget_events->removeRow(row);

    _dontUndo = false;
    updateUndoList();
}

void EventSelectionAssistant::on_toolButton_nextImages_clicked()
{
    _lastSkippedTo = QDateTime();
    _setNextUncoveredTime();
}

void EventSelectionAssistant::on_toolButton_showImages_clicked()
{
    const int row = _getSelectedRow();
    if (row < 0) return;

    _currFrom = QDateTime::fromTime_t(ui->tableWidget_events->item(row,1)->data(Qt::DisplayRole).toUInt());
    _currTo = QDateTime::fromTime_t(ui->tableWidget_events->item(row,2)->data(Qt::DisplayRole).toUInt());
    displayDate(true);
}

void EventSelectionAssistant::on_toolButton_updateCurrImages_clicked()
{
    ui->dateTimeEdit_newEventFrom->interpretText();
    ui->dateTimeEdit_newEventTo->interpretText();

    if (!ui->dateTimeEdit_newEventFrom->date().isValid()
            || !ui->dateTimeEdit_newEventTo->date().isValid()) {

        QMessageBox::warning ( this,
                               tr("Date invalid"),
                               tr("Please enter valid dates and times for this event"));
        return;
    }

    _currFrom = ui->dateTimeEdit_newEventFrom->dateTime();
    _currTo = ui->dateTimeEdit_newEventTo->dateTime();
    displayDate();

    updateUndoList();
}


void EventSelectionAssistant::onTimeFromForNewTriggered(const QDateTime& newDate)
{
    ui->dateTimeEdit_newEventTo->interpretText();
    _currTo = ui->dateTimeEdit_newEventTo->dateTime();
    if (false) { //_currTo < newDate) {
        QMessageBox::critical ( this,
                               tr("Date invalid"),
                               tr("new end date of album earlier than start date"));
        return;
    }


    ui->dateTimeEdit_newEventFrom->setDateTime(newDate);
    _currFrom = newDate;

    // wait for finish of this signal by pushing the displaydate call to the back of the event queue
    QTimer::singleShot(0, this, SLOT(displayDate()));

    updateUndoList();
}

void EventSelectionAssistant::onTimeToForNewTriggered(const QDateTime& newDate)
{
    ui->dateTimeEdit_newEventFrom->interpretText();
    _currFrom = ui->dateTimeEdit_newEventFrom->dateTime();

    if (false) { //_currFrom > newDate) {
        QMessageBox::critical ( this,
                               tr("Date invalid"),
                               tr("new start date of album later than end date"));
        return;
    }

    ui->dateTimeEdit_newEventTo->setDateTime(newDate.addSecs(1));
    _currTo = newDate.addSecs(1);

    // wait for finish of this signal by pushing the displaydate call to the back of the event queue
    QTimer::singleShot(0, this, SLOT(displayDate()));

    updateUndoList();
}

void EventSelectionAssistant::onTimeFromForTableTriggered(const QDateTime& newDate)
{    
    const int row = _getSelectedRow();
    if (row < 0 || row >= ui->tableWidget_events->rowCount())
        return;

    ui->tableWidget_events->item(row, 1)->setData(Qt::DisplayRole, newDate.toTime_t() + (uint)1);
    _currTo = QDateTime::fromTime_t(ui->tableWidget_events->item(row, 2)->data(Qt::DisplayRole).toUInt());
    if (false) { //_currTo < newDate) {
        QMessageBox::critical ( this,
                             tr("Date invalid"),
                             tr("new end date of album earlier than start date"));
        return;
    }

    _currFrom = newDate;
    //qApp->processEvents();

    // wait for finish of this signal by pushing the displaydate call to the back of the event queue
    QTimer::singleShot(0, this, SLOT(displayDate()));

    updateUndoList();
}

void EventSelectionAssistant::onTimeToForTableTriggered(const QDateTime& newDate)
{
    const int row = _getSelectedRow();
    if (row < 0 || row >= ui->tableWidget_events->rowCount()) return;

    ui->tableWidget_events->item(row, 2)->setData(Qt::DisplayRole, newDate.toTime_t() + (uint)1);
    _currFrom = QDateTime::fromTime_t(ui->tableWidget_events->item(row, 1)->data(Qt::DisplayRole).toUInt());

    if (false) { //_currFrom > newDate) {
        QMessageBox::critical ( this,
                               tr("Date invalid"),
                               tr("new start date of album later than end date"));
        return;
    }

    _currTo = newDate.addSecs(1);

    // wait for finish of this signal by pushing the displaydate call to the back of the event queue
    QTimer::singleShot(0, this, SLOT(displayDate()));
}


int EventSelectionAssistant::_getSelectedRow(const bool warning)
{
    QList<QTableWidgetSelectionRange> selection = ui->tableWidget_events->selectedRanges();
    if (warning && (selection.size() == 0 || selection.size() > 1)) {
        QMessageBox::warning(this,
                             tr("Nothing selected"),
                             tr("Please select an album to display"));
        return -1;
    } else if (selection.size() == 1){
        return selection[0].topRow();
    } else {
        return -1;
    }
}

void EventSelectionAssistant::updateUndoList()
{
    if (!_dontUndo) {
        if (_undoPosition < _undoHistory.size() -1) {
            _undoHistory.erase( _undoHistory.begin() + _undoPosition + 1,
                                _undoHistory.end());
        }

        UndoItem newUndo;
        newUndo.albums = getAlbums();
        newUndo.currDateFrom = _currFrom;
        newUndo.currDateTo = _currTo;

        _undoHistory.push_back(newUndo);
        _undoPosition = _undoHistory.size()-1;
        ui->toolButton_redo->setEnabled(false);
        ui->toolButton_undo->setEnabled(_undoHistory.size() > 1 ? true : false);
    }
}

void EventSelectionAssistant::_setCurrentUndo() {
    if (_undoPosition < (int)_undoHistory.size() && _undoPosition >=0) {
        const UndoItem& undo = _undoHistory.at(_undoPosition);
        _setAlbums(undo.albums);
        _currFrom = undo.currDateFrom;
        _currTo = undo.currDateTo;
        displayDate();
    }
}

void EventSelectionAssistant::on_toolButton_undo_clicked()
{
    _dontUndo = true;
    _undoPosition--;
    if (_undoPosition <= 0) {
        _undoPosition = 0;
        ui->toolButton_undo->setEnabled(false);
    }

    if (_undoPosition < (int)_undoHistory.size() -1) {
        ui->toolButton_redo->setEnabled(true);
    }

    _setCurrentUndo();

    _dontUndo = false;
}

void EventSelectionAssistant::on_toolButton_redo_clicked()
{
    _dontUndo = true;
    _undoPosition++;
    if (_undoPosition >= (int)_undoHistory.size() -1) {
        _undoPosition = _undoHistory.size()-1;
        ui->toolButton_redo->setEnabled(false);
    }

    if (_undoPosition > 0) {
        ui->toolButton_undo->setEnabled(true);
    }

    _setCurrentUndo();

    _dontUndo = false;
}

void EventSelectionAssistant::itemSelectionChanged()
{
    const int row = _getSelectedRow(false);
    if (row < 0) {
        emit tableViewSelected(false);
    } else {
        emit tableViewSelected(true);
    }
}

bool EventSelectionAssistant::eventFilter(QObject *object, QEvent *event)
{

    if (object == ui->lineEdit_newEventDescription
            && event->type() == QEvent::KeyPress
            && ( static_cast<QKeyEvent*>(event)->key() == Qt::Key_Return
                 || static_cast<QKeyEvent*>(event)->key() == Qt::Key_Enter)
            )
    {
        return true;
    } else if (object == ui->tableWidget_events
               && event->type() == QEvent::KeyPress
               && ( static_cast<QKeyEvent*>(event)->key() == Qt::Key_Return
                    || static_cast<QKeyEvent*>(event)->key() == Qt::Key_Enter)

               )
    {
        return true;
    } else if (event->type() == QEvent::FocusIn) {
        // clear table selection if new album fields become focused
        // but return false to have the current item handle the input correctly
        if (object == ui->dateTimeEdit_newEventFrom
                || object == ui->lineEdit_newEventDescription
                || object == ui->dateTimeEdit_newEventTo)
        {
            ui->tableWidget_events->clearSelection();
            return false;
        }
    }
    if (event->type() == QEvent::MouseButtonPress) {
        if (object == ui->tableWidget_events
                && static_cast<QMouseEvent*>(event)->button() == Qt::MiddleButton )
        {
            const int row = _getSelectedRow(false);
            if (row >= 0) {
                QTimer::singleShot(0, this, SLOT(on_toolButton_showImages_clicked()));
            }
        }
    }
    return QObject::eventFilter(object, event);
}

void EventSelectionAssistant::resetData(const QString& location)
{
    _settingsLocation = QDir::toNativeSeparators( location + QDir::separator() + ALBUMS_DATA_FILE );
    QFile::remove(_settingsLocation);
    _setAlbums(QVector<PhotoAlbum>());
}

void EventSelectionAssistant::_writeAlbumsToFile()
{
    if (_settingsLocation.length() == 0) {
        return;
    }
    QFile file(_settingsLocation);
    file.open(QIODevice::WriteOnly);
    if (!file.isOpen()) {
        qCritical() << "could not save album data";
    }
    QDataStream out(&file);

    // Write a header with a "magic number" and a version
    out << (quint32)ALBUMS_FILE_NUMBER;
    out << (qint32)QDataStream::Qt_4_7;

    out.setVersion(QDataStream::Qt_4_7);

    // write the current id
    out << _lastID;

    // Write the data
    out << getAlbums();
}

void EventSelectionAssistant::loadAlbumsFromFile(const QString& location)
{
    _settingsLocation = QDir::toNativeSeparators( location + QDir::separator() + ALBUMS_DATA_FILE );


    QFile file(_settingsLocation);
    if (!file.open(QIODevice::ReadOnly)) {
        return;
    }
    QDataStream in(&file);

    // Read and check the header
    quint32 magic;
    in >> magic;
    if (magic != ALBUMS_FILE_NUMBER) {
        QMessageBox::critical(this,
                              tr("could not load albums"),
                              tr("Could not load saved albums - wrong file version number"));
                return;
    }

    // Read the version
    qint32 version;
    in >> version;

    in.setVersion(version);

    // read the current id
    in >> _lastID;

    // Read the data
    QVector<PhotoAlbum> albums;
    in >> albums;

    for (int i=0; i<albums.size(); i++) {
        if (!albums[i].from.isValid() || !albums[i].to.isValid()) {
            qWarning() << "found INVALID album " << albums[i].name;
        }
    }

    _dontUndo = true;

    _setAlbums(albums);
    _dontUndo = false;
    updateUndoList();
}

void EventSelectionAssistant::_viewAlbums(
        const QList<ImageDataPtr>& imageList,
        const QString& path)
{
    ViewImageDialog* viewImages = ViewImageDialogExtended::instance(this->parentWidget());
    viewImages->setModal(false);
    viewImages->show();
    viewImages->activateWindow();

    int row = 0;
    for (int i=0; i<(int)imageList.size(); i++) {
        if (imageList[i]->getFileInfo().canonicalFilePath() == path) {
            row = i;
        }
    }

    viewImages->setImages(imageList,
                          row,
                          _baseDirLength);

}

void EventSelectionAssistant::_viewAllImages(const QString &path)
{
    //qApp->processEvents();
    _viewAlbums(_photoList, path);
}

void EventSelectionAssistant::_viewCurrentAlbum(const QString &path)
{
    QList<ImageDataPtr> showImages;

    for (int i=0; i<(int)_photoList.size(); i++) {
        ImageDataExtended* tmp = static_cast<ImageDataExtended*>(_photoList[i].get());
        if (tmp->getTime() >= _currFrom && tmp->getTime() < _currTo) {
            showImages.push_back(_photoList[i]);
        }
    }

    _viewAlbums(showImages, path);
}

void EventSelectionAssistant::showInfo()
{
    if (!_infoHasBeenShown) {
        if (!MiscTools::MiscTools::lockDialog()) {
            QTimer::singleShot(10000, this, SLOT(showInfo()));
            return;
        }
        _infoHasBeenShown = true;
        Tools::provideInfo(this,
                           tr("Album Management"),
                           tr("<i>This is the album management dialog.</i><br><br>"
                              "Here you can edit the albums which will serve as name basis for copying. "
                              "Albums can be edited/created by means of:<ul>"
                              "<li>selecting images which are not yet in albums</li>"
                              "<li>right clicking on images in the preview to use "
                              " their dates as start or ending times (context menu)</li>"
                              "</ul>From the current period some(or all) images will"
                              " be displayed, as well as some(or all) images "
                              " right before and right after."),
                           tr("editalbumsview"));
        MiscTools::MiscTools::unlockDialog();
    }
}

void EventSelectionAssistant::on_toolButton_clicked()
{
    const QMessageBox::StandardButton answer =
            QMessageBox::question(this,
                                  tr("Delete all albums ?"),
                                  tr("Really delete all albums ?"),
                                  QMessageBox::No | QMessageBox::Yes);
    if (answer == QMessageBox::No) {
        return;
    }

    _setAlbums(QVector<PhotoAlbum>());
}
