/***********************************************************************
 * This file is part of pivot.
 *
 * pivot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pivot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with pivot.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2015, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#ifndef IMAGEWIDGET_H
#define IMAGEWIDGET_H

#include <QWidget>
#include <QAction>

#include <imagedataextended.h>

namespace Ui {
    class ImageWidget;
}

class ImageWidget : public QWidget
{
    Q_OBJECT

public:
    explicit ImageWidget(const ImageDataExtended photo,
                         const int thumbnailSize,
                         const bool showAlbumView,
                         const bool currSelected,
                         QWidget *parent = 0);
    ~ImageWidget();

    ImageDataExtended getImageDataExtented() { return _photo; }

signals:
    void signalTimeFromForNew(const QDateTime);
    void signalTimeToForNew(const QDateTime);
    void signalTimeFromForTable(const QDateTime);
    void signalTimeToForTable(const QDateTime);

    void signalViewAllImages(const QString& path);
    void signalViewCurrentAlbum(const QString& path);

public slots:
    void updateThumbnail(const MiscTools::pQImage &thumbnail);
    void updateThumbnailSize(const int thumbnailSize);

    void setTableSelected(const bool enabled);

private slots:
    void displayThumbnail();

private slots:
    void onTimeFromForNewTriggered();
    void onTimeToForNewTriggered();
    void onTimeFromForTableTriggered();
    void onTimeToForTableTriggered();

    void onViewAllImages();
    void onViewCurrentAlbum();

private:
    QAction* timeFromForNew;
    QAction* timeToForNew;
    QAction* timeFromForTable;
    QAction* timeToForTable;

    QAction* viewAllImages;
    QAction* viewCurrentAlbum;

    Ui::ImageWidget *ui;
    ImageDataExtended _photo;
    int _thumbnailSize;
};

#endif // IMAGEWIDGET_H
