/***********************************************************************
 * This file is part of pivot.
 *
 * pivot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pivot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with pivot.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2015, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#include <QApplication>
#include <QtGlobal>
#include <QFileInfo>
#include <QMetaType>
#include <QTimer>
#include <QTranslator>
#include <QFileInfo>
#include <QDir>
#include <QFile>
#include <QTextStream>
#include <QMessageBox>
#include <QWidget>

//#include "C:\Program Files (x86)\Visual Leak Detector\include\vld.h"

#include "mainwindow.h"
#include "eventselectionassistant.h"


#include <iostream>
#include <fstream>

#include "imagedataextended.h"
#include "eventselectionassistant.h"
#include <qtsingleapplication.h>
#include "viewimagedialog.h"
#include "version_pivot.h"
#include "debugtofile.h"
#include "copyfiles.h"
#include "translation.h"


int main(int argc, char** argv)
{
#ifdef _WIN32
	// for some reason, qwindows.dll doesn't load if it is started from another directory
	const QString locationExe = QFileInfo(QCoreApplication::applicationFilePath()).canonicalPath();
	QDir x(locationExe);
	x.cd(locationExe);
	QCoreApplication::addLibraryPath("./");
	QCoreApplication::addLibraryPath("./bin/");
	QCoreApplication::addLibraryPath(locationExe);
#endif

    QtSingleApplication instance("pivot - Photo, Image and Video Organization Tool", argc, argv);

    // ### redirect qDebug, qWarning, etc... to a file
	QString location = MiscTools::MiscTools::appDataPath();
    QDir locationDir(location);
    const QString _settingsLocation = QDir::toNativeSeparators(location + QDir::separator() + SETTINGS_FOLDER + QDir::separator());
    locationDir.mkpath(_settingsLocation);
#ifdef _WIN32
	const QString debugFilename = _settingsLocation + QDir::separator() + "debugPivot.txt";    
#else
    const QString debugFilename = _settingsLocation + QDir::separator() + "debugPivot";
#endif
    // remove file head, if too large
    const quint64 s = QFileInfo(debugFilename).size(), maxSize = 10000000;
    if (s > maxSize) {
        QFile file(debugFilename);
        QStringList lines;
        if(file.open(QIODevice::ReadOnly)) {
            QTextStream in(&file);
            while(!in.atEnd()) {
                QString line = in.readLine();
                lines.append(line);
            }
            file.close();
            if (file.open(QIODevice::WriteOnly)) {
                QTextStream out(&file);
                for (auto i=lines.size()-1; i>= lines.size()/2; i--) {
                    out << lines[i] << "\n";
                }
            }
        }
    }
    MiscModule::setDebugFilename(debugFilename);
    qInstallMessageHandler(MiscModule::customMessageHandler);
	
    // ### setup warranty display
    QFile file(":/WARRANTY");
    std::cout << "This is pivot - Photo, Image and Video Organization Tool - version "
              << version_pivot.getVersionMajor() << "."
              << version_pivot.getVersionMinor() << "."
              << version_pivot.getVersionPatch()
              << std::endl;

    if (file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QTextStream in(&file);
        while (!in.atEnd()) {
            std::cout << in.readLine().toStdString() << std::endl;
        }
        std::cout << std::endl;
    } else {
        std::cout << "not found" << std::endl;
    }
    file.close();

	// debug all arguments
    QStringList args = instance.arguments();
	for (int i=args.length()-1; i>0; i--) {
		qDebug() << "args: " << args[i];
	}
	
        // ### load locale
        QString locale = QLocale::system().name();
        for (int i=args.length()-1; i>0; i--) {
            if (args[i]== "-l" || args[i] == "--locale") {
                if (argc <= i+1) {
                    qWarning() << QString("please provide a locale, e.g.: %1 -l en").arg(args[0]);

                } else {
                    locale = args[i+1];
                    qDebug() << "trying to load locale " << locale << "...";
                    args.removeAt(i+1);
                }
                args.removeAt(i);
            }
        }

        // set locale and load translations
        MiscTools::Translation::setDefaultLocale(locale);

        QStringList langPaths = QStringList() << QString(":/module_imageviewer/")
                                              << QString(":/module_progressdialog/")
                                              << QString(":/")
                                              << QString(":/module_imagepreview/");
        instance.installTranslator(MiscTools::Translation::getTranslator(locale, langPaths, "qt_"));
        instance.installTranslator(MiscTools::Translation::getTranslator(locale, langPaths, "trans_pivot_"));
        instance.installTranslator(MiscTools::Translation::getTranslator(locale, langPaths, "trans_imageviewer_"));
        instance.installTranslator(MiscTools::Translation::getTranslator(locale, langPaths, "trans_progressdialog_"));
        instance.installTranslator(MiscTools::Translation::getTranslator(locale, langPaths, "trans_imagepreview_"));

    // initialize tags AFTER loading language
    CopyFiles::initializeTags();

    // ### load provided directory
    QString message;
	bool foundDevice = false;
#ifdef _WIN32
	// parse for windows camera import
	for (int i=args.length()-1; i>0; i--) {
        if (args[i]== "--wpd-device") {
			foundDevice = true;
            if (argc <= i+1) {
                qWarning() << QString("no device provided").arg(args[0]);
            } else {
                const QString deviceDesc = args[i+1];
                const int p = deviceDesc.indexOf("\\");
                qDebug() << "device description: " << deviceDesc;
                if (p > 0) {
                    const QString deviceID = deviceDesc.left(p);
                    const QString deviceNumber = deviceDesc.right(deviceDesc.length() - p -1);
                    qDebug() << "device id: " << deviceID;
                    qDebug() << "device number: " << deviceNumber;
                    message = MiscTools::MiscTools::getDevicePath(deviceID, deviceNumber);
                    qDebug() << "device path: " << message;
                }
            }
            args.removeAt(i);
            if (foundDevice) break;
        }
	}
#endif
    if (!foundDevice && message.length() == 0 && args.size() > 1) {
        message = args[1];
    }

    if (message.length() > 0 ) {
        qDebug() << "trying to load directory " << message << " ...";
    }

    if (instance.sendMessage(message))
        return 0;

    qRegisterMetaType<QFileInfo>("QFileInfo");
    qRegisterMetaType<ImageData>("ImageData");
    qRegisterMetaType<ImageDataExtended>("ImageDataExtended");
    qRegisterMetaType<PhotoAlbum>("PhotoAlbum");
    qRegisterMetaType<QVector<PhotoAlbum> >("QVector<PhotoAlbum>");
    qRegisterMetaType<MiscTools::pQImage>("MiscTools::pQImage");
    qRegisterMetaType<MiscTools::pQPixmap>("MiscTools::pQPixmap");
    qRegisterMetaTypeStreamOperators<PhotoAlbum>("PhotoAlbum");
    qRegisterMetaTypeStreamOperators<QVector<PhotoAlbum> >("QVector<PhotoAlbum>");
    Q_INIT_RESOURCE(imageviewericons);
    Q_INIT_RESOURCE(iconsthumb);
    Q_INIT_RESOURCE(infoimages);

    const QIcon icon(":/images/PhotoOrganizer.ico");
    QApplication::setWindowIcon(icon);

    MainWindow mw;
    mw.show();
    qApp->processEvents();
    if (message.length() > 0 ) {
        mw.loadDirectory(message);
    }

    QObject::connect(&instance, SIGNAL(messageReceived(const QString&)),
                     &mw, SLOT(loadDirectory(QString)));

    instance.setActivationWindow(&mw, false);

//    foreach (QWidget *widget, QApplication::allWidgets()) {
//        if (dynamic_cast<QLabel*>(widget)) {
//            qDebug() << widget << ": " << widget->styleSheet();
//        }
//    }

    return instance.exec();

}
