/***********************************************************************
 * This file is part of pivot.
 *
 * pivot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pivot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with pivot.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2015, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#ifndef MAINSETTINGS_H
#define MAINSETTINGS_H

#include <QDialog>
#include <QStringList>
#include <QToolButton>
#include <QLineEdit>
#include <QGridLayout>
#include <QSpacerItem>

namespace Ui {
    class MainSettings;
}

struct DestinationSettingsOption
{
    QLineEdit* destinationDescription;
    QToolButton* remove;
    QToolButton* add;
};

enum CopyOverwriteMode
{
    CopyOverwrite,
    CopyNoOverwrite,
    CopyAsk
};

class MainSettings : public QDialog
{
    Q_OBJECT

public:
    explicit MainSettings(QWidget *parent = 0);
    ~MainSettings();

    int getThumbnailSize() const;
    bool tryToObtainByBasename() const;
    bool tryToObtainByNeighbour() const;
    bool tryToObtainFileTime() const;
    bool tryUseMetaData() const;
    QStringList getExtensions() const;
    QStringList getRawExtensions() const;
    bool tryUpdateTimeMetaData() const;
    bool saveToTextFileIfFailure() const;
    QStringList getDestinationDescriptions() const;
    bool getUseUniqueFiles() const;
    bool getCombineRAWAndImage() const;
    bool getCopyOnlyOnce() const;
    bool getSaveAlbumDate() const;
    bool getDifferentTags() const;
    QString getTagsKnownDay() const;
    QString getTagsUnknownDay() const;
    QString getTagsKnownDayAlbum() const;
    QString getTagsUnknownDayAlbum() const;
    bool getUseComments() const;
    bool getDoAutosave() const;
    int getAutosaveInterval() const;
    CopyOverwriteMode getOverwriteMode() const;
    bool getForceSameNumber() const;
    bool getExistingFilesMerge() const;
    bool getExistingFilesContinue() const;
    bool getExistingFilesContinueRename() const;
    bool getCreateOwnerShipText() const;

    void checkNewVersionAndDeleteSettings(const int major,
                                          const int minor,
                                          const int patch) ;

signals:
    void resetSettings();
    void resetUserData();

private slots:
    void on_buttonBox_rejected();

    void on_buttonBox_accepted();

    void _removeDestinationRow();
    void _addDestinationRow();

    void on_pushButton_clicked();

    void on_pushButton_2_clicked(const bool preventDialog = false);

    void on_toolButton_whatsThis_clicked();

    void on_radioButton_merge_toggled(bool checked);

    void on_radioButton_continue_toggled(bool checked);

    void on_checkBox_copyonce_toggled(bool checked);

private:
    void _loadSettings();
    void _saveSettings();

    void _duplicateMessage() ;

    DestinationSettingsOption _insertDestination(const QString& text,
                                                 const int row,
                                                 QGridLayout* layout);

    Ui::MainSettings *ui;

    QVector<DestinationSettingsOption> _destinations;

    QSpacerItem* _spacerDestinations;

    bool _init;
};

#endif // MAINSETTINGS_H
