/***********************************************************************
 * This file is part of pivot.
 *
 * pivot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pivot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with pivot.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2015, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#include <QTimer>
#include "previewdialog.h"
#include "ui_previewdialog.h"

#include <QWhatsThis>

#include <tools.h>

PreviewDialog::PreviewDialog(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::PreviewDialog)
    , _infoHasBeenShown(false)
{
    ui->setupUi(this);
    this->setWindowTitle(tr("(Preview) Copy Results"));
}

void PreviewDialog::setText(const QString& text)
{
    ui->plainTextEdit->setPlainText(text);
}

PreviewDialog::~PreviewDialog()
{
    delete ui;
}

void PreviewDialog::showInfo()
{
    if (!_infoHasBeenShown) {
        if (!MiscTools::MiscTools::lockDialog()) {
            QTimer::singleShot(10000, this, SLOT(showInfo()));
            return;
        }
        _infoHasBeenShown = true;
        Tools::provideInfo(this,
                           tr("Preview of Copying"),
                           tr("<i>This is the copy preview dialog.</i><br><br>"
                              "Here you will see the preview of the copying "
                              "result. Note that there exists a variety of "
                              "reasons why a file will not be copied. <b>Make "
                              "sure you check the preview for these erroneous "
                              "files</b>."),
                           tr("previewofcopying"));
        MiscTools::MiscTools::unlockDialog();
    }
}
