/***********************************************************************
 * This file is part of pivot.
 *
 * pivot is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * pivot is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with pivot.  If not, see <http://www.gnu.org/licenses/>
 * 
 * 
 * Copyright (C) 2015, Dominik Rueß; info@dominik-ruess.de
 **********************************************************************/

#ifndef VIEWMODEL_H
#define VIEWMODEL_H

#include "imagedataextended.h"
#include "imageviewmodel.h"

#include "eventselectionassistant.h"

#define IMAGE_VIEW_NUM_COLS 8
#define IMAGE_VIEW_NUM_COLS_WO_DETAILS 4

class ViewModel : public ImageViewModel
{
    Q_OBJECT

public:
    explicit ViewModel(QObject *parent = 0);

    int rowCount(const QModelIndex& parent) const;
    int columnCount(const QModelIndex& parent) const;
    QVariant data(const QModelIndex& index, int role) const;
    QVariant headerData(int section, Qt::Orientation orientation, int role) const;

    bool setData ( const QModelIndex & index, const QVariant & value, int role = Qt::EditRole );
    Qt::ItemFlags flags ( const QModelIndex & index ) const;

    void setShowDetails(const bool);
    void setShowCrop(const bool);


    const ImageDataExtended& getImageDataExtended(const int pos) const;

    void setList(const QList<ImageDataPtr>&);
    void updateList(const int pos, const ImageDataExtended&);

    void setRootDirectory(const QString& dir) { rootDirectory = dir; }

    void setUseDifferentTags(const bool use);

    void setUseComments(const bool useComments);

    void updateFilesAlbums(const QVector<PhotoAlbum>& albums);

    QVector<int> getImageTimeSortIndexes(const QVector<int> indexes = QVector<int>()) const;

signals:
    void errorMessage(const QString& headline, const QString& errorMsg);

public slots:
    void toggleNoTimeOrAlbum(const bool);

    void timeDown(const QVector<int> rows);
    void timeUp(const QVector<int> rows);
    void timeReset(const QVector<int> rows);

    void albumDown(const QVector<int> rows);
    void albumUp(const QVector<int> rows);
    void albumReset(const QVector<int> rows);

    void destinationDown(const QVector<int> rows);
    void destinationUp(const QVector<int> rows);
    void destinationReset(const QVector<int> rows);

    void setNumDestinations(const int num);

private:
    bool _isCommentColumn(const int index) const;

    bool _isStatusColumn(const int index) const;

    bool _isTargetColumn(const int index) const;

    void _computeNumMissing();

    static MiscTools::pQImage _noThumb;
    static MiscTools::pQImage _errorThumb;

    bool _showDetails;
    bool _showCrop;

    QFileInfo rootDirectory;

    bool _showOnlyMissing;
    int _numMissing;
    QVector<int> _backMap;

    QVector<PhotoAlbum> _albums;

    QDateTime _lastTime;

    bool _useDifferentTags;

    bool _useComments;

    int _numCopyDestinations;
};

#endif // VIEWMODEL_H
